Unit E_Props;
{-------------} Interface {--------------------}
Uses
  Consts,
  Classes  ,TypInfo, Windows, Forms,SysUtils,
  Controls,Db,DbTables,Graphics,Dialogs;
(*******************************************************************
                            E_Props
    Get and set RTTI information routines.
    NOTE : I have encapsulated all RTTI manipulation routines in this
    unit due to the possibility that Borland may change the structure
    of RTTI with later releases. This way any changes required in an
    application can be made in only one place. The TEProperty class
    below provides an interface that can be used for displaying and
    editing a component's properties.
    Author : David Spies & Jan TUNGLI
    Contacts : Work - davidsp@eastsoft.com Home DSPIES@onecom.com
             : jan.tungli@seznam.cz  Home:http://www.tsoft.szm.com 
                                          
    *)

Const
       c_MaxSize=32000;
       PROP_NOTYPE        = 0;  //?
       PROP_STRTYPE       = 1;
       PROP_INTTYPE       = 2;  //
       PROP_REALTYPE      = 3;  //
       PROP_BOOLTYPE      = 4;  //
       PROP_CHARTYPE      = 5;
       PROP_ENUMTYPE      = 6;
       PROP_COLORTYPE     = 7;
       PROP_CURSORTYPE    = 8;
       PROP_SETTYPE       = 9;
       PROP_CLASSTYPE     = 10;
       PROP_MODALTYPE     = 11;
       PROP_DBNAMETYPE    = 12;
       PROP_DBIDXNAMETYPE = 13;
       PROP_DBTABNAMETYPE = 14;
       PROP_DBLOOKUPFIELD = 15;
       PROP_INDEXNAME     = 16;

       PROP_COMPONENT     = 30;  //common component - obecna komponenta

       PROP_FONTSUB        = 1;
       PROP_ICONSUB        = 2;
       PROP_BMPSUB         = 3;
       PROP_TSTRSUB        = 4;
       PROP_DATASETSUB     = 5;
       PROP_DATASOURCESUB  = 6;
Type
     TEProperty = Class
       EType        : Word;
       SubType      : Word;
       MaxChars     : Word;
       MinVal       : LongInt;
       MaxVal       : LongInt;
       ClassAddr    : LongInt;
       PValue       : String;
       TypeInfo     : PPTypeInfo;
       MyClassName  : string; // tungli
     end;

Type
       ECompEditError   = class(Exception);
Const
       MrArray   : Array[mrNone..MrNo] Of String =
        ('mrNone',
         'mrOk',
         'mrCancel',
         'mrAbort',
         'mrRetry',
         'mrIgnore',
         'mrYes',
         'mrNo');

Function StripAllChr(InStr    : String;
                     StripChr : Char) : String;

Function E_IsPublishedProp(     AComponent : TComponent;
                              PropName   : String) : Boolean;

Function E_GetStrProp(      AComponent : TComponent;
                            PropName   : String;
                      Var   PropValue  : String) : Boolean;

Function E_GetIntProp(      AComponent : TComponent;
                            PropName   : String;
                      Var   PropValue  : Integer) : Boolean;

Function E_GetBoolProp(      AComponent : TComponent;
                             PropName   : String;
                       Var   PropValue  : Boolean) : Boolean;

Function E_GetRealProp(     AComponent : TComponent;
                            PropName   : String;
                      Var   PropValue  : Double) : Boolean;

Function E_GetSetStrProp(      AComponent : TComponent;
                               PropName   : String;
                          Var  PropValue  : String) : Boolean;

Function E_SetStrProp(      AComponent : TComponent;
                            PropName   : String;
                      Const PropValue  : String) : Boolean;

Function E_SetIntProp(      AComponent : TComponent;
                            PropName   : String;
                            PropValue  : Integer) : Boolean;

Function E_SetRealProp(      AComponent : TComponent;
                            PropName   : String;
                            PropValue   : Double) : Boolean;

Function E_SetBoolProp(     AComponent : TComponent;
                            PropName   : String;
                            PropValue  : Boolean) : Boolean;

Function E_SetSetStrProp(      AComponent : TComponent;
                               PropName   : String;
                               PropValue  : String) : Boolean;

Function E_EnumProperties( AComponent : TComponent;
                          Var PropList: TStringList;
                          pSubProp    : string) : Integer;

procedure E_GetComponent(var AComponent : TComponent; var PropName: String);
function  FindComp(AOwnerForm:TComponent; CompName:String):TComponent;
Procedure GetPropValue(AComponent : TComponent; PropName:string; var CompProp:TEProperty);
function GetPropValueStr(AComponent : TComponent; PropName:string):string;
procedure SetPropValue(AComponent:TComponent; PropName:string;xValue:string);

function GetProp(PropName:string; DefaultFormName: string='SELF'):string;
procedure SetProp(PropName,xValue:string; DefaultFormName:string='SELF');
function ExistObj(ObjName:string;DefaultFormName:string='SELF'):boolean;
function ExistProp(PropName:string; DefaultFormName:string='SELF'):boolean;
function ParseProp(DefaultForm:TForm; var PropName:string):TComponent;
function ClassNameObj(ObjName:string;DefaultFormName:string='SELF'):string;
function GetComponent(ObjName:string;DefaultFormName:string='SELF'):Tcomponent;


Function E_GetEnumList(      CompProp : TEProperty;
                             AList    : TStrings) : Boolean;

function CompExist(pClassName:string):boolean;
function LastPropType:word;
{-------------} Implementation {---------------}



Type
     TIntegerSet = set of 0..SizeOf(Integer) * 8 - 1;

var MyLastPropType:word;

function LastPropType:word;
begin
  Result:=MyLastPropType;
end;

Function StripAllChr(InStr    : String;
                     StripChr : Char) : String;
Var
  I : Integer;
begin
  I := 1;
  while (I <= Length(InStr)) do
    if InStr[I] = StripChr then
      Delete(InStr, I, 1)
    else
      Inc(I);
  Result := InStr;
end;

(*******************************************************************
                            E_GetPropInfo
    Internal helper routine to get a RTTI structure pointer.
 *******************************************************************)
Function E_GetPropInfo(      AComponent : TCOmponent;
                       Const PropName   : String;
                       Var   PropInfo   : PPropInfo) : Boolean;
begin
  PropInfo:=Nil;
  If AComponent<>Nil then
    Try
      PropInfo:=GetPropInfo(AComponent.ClassInfo,PropName);
    Except
      PropInfo:=Nil;
    end;
  Result:=PropInfo<>Nil;
end;

procedure E_GetComponent(var AComponent : TComponent;
                         var PropName   : String);
// PropName='Font.Color'
Var
    PropInfo  : PPropInfo;
    mAddr     : longint;
    i         : integer;
    mT        : string;
begin
  while Pos('.',PropName)>0 do begin
    i:=Pos('.',PropName);
    mT:=copy(PropName,1,i-1);
    E_GetPropInfo(AComponent,mT,PropInfo);
    mAddr:=GetOrdProp(AComponent,PropInfo);
    AComponent:=TComponent(mAddr);
    delete(PropName,1,i);
  end;
end;

(*******************************************************************
                            E_IsPublished
    Return true if PropName is a published property.
    enabled SubProp
 *******************************************************************)
Function E_IsPublishedProp(     AComponent : TComponent;
                                PropName   : String) : Boolean;
begin
  E_getComponent(AComponent,PropName);
  If Propname<>'' then
    Result:=GetPropInfo(AComponent.ClassInfo,PropName)<>Nil
  else
    Result:=False;
end;

(*******************************************************************
                            E_Get????????
  The next several routines get the value of PropName from component.
  Returns False if property doesn't exist.
  enabled SubProp
 *******************************************************************)

Function E_GetStrProp(      AComponent : TComponent;
                            PropName   : String;
                      Var   PropValue  : String) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    PropValue:=GetStrProp(TObJect(AComponent),PropInfo)
  else
    PropValue:='';
end;
Function E_GetIntProp(      AComponent : TComponent;
                            PropName   : String;
                      Var   PropValue  : Integer) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    PropValue:=GetOrdProp(TObJect(AComponent),PropInfo)
  else
    PropValue:=0;
end;
Function E_GetBoolProp(      AComponent : TComponent;
                             PropName   : String;
                       Var   PropValue  : Boolean) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    PropValue:=Boolean(GetOrdProp(TObJect(AComponent),PropInfo))
  else
    PropValue:=False;
end;
Function E_GetRealProp(      AComponent : TComponent;
                             PropName   : String;
                       Var   PropValue  : Double) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    PropValue:=GetFloatProp(TObJect(AComponent),PropInfo)
  else
    PropValue:=0;
end;
Function E_GetSetStrProp(      AComponent : TComponent;
                               PropName   : String;
                          Var  PropValue  : String) : Boolean;
Var
    PropInfo  : PPropInfo;
    S         : TIntegerSet;
    TypeInfo  : PPTypeInfo;
    I         ,
    MinV      ,
    MaxV      : Integer;
begin
  E_getComponent(AComponent,PropName);
  PropValue:='';
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
  begin
    Integer(S):=GetOrdProp(TObJect(AComponent),PropInfo);
    TypeInfo:=GetTypeData(PropInfo^.PropType^).CompType;
    MinV:=GetTypeData(TypeInfo^).MinValue;
    MaxV:=GetTypeData(TypeInfo^).MaxValue;
    PropValue := '[';
    for I := MinV to MaxV do
      if I in S then
      begin
        if Length(PropValue) <> 1 then
          PropValue := PropValue + ',';
        PropValue := PropValue + GetEnumName(TypeInfo^,I);
      end;
      PropValue := PropValue + ']';
  end;
end;
(*******************************************************************
                            E_Set????????
  The next several routines sets the value of PropName in component
  to PropVal. Returns False if property doesn't exist.
  enabled SubProp
 *******************************************************************)
Function E_SetStrProp(      AComponent : TComponent;
                            PropName   : String;
                      Const PropValue  : String) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    SetStrProp(TObJect(AComponent),PropInfo,PropValue);
end;
Function E_SetIntProp(      AComponent : TComponent;
                            PropName   : String;
                            PropValue  : Integer) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    SetOrdProp(TObJect(AComponent),PropInfo,PropValue);
end;
Function E_SetBoolProp(     AComponent : TComponent;
                            PropName   : String;
                            PropValue  : Boolean) : Boolean;
Var
    PropInfo  : PPropInfo;
    PValue    : Integer;
begin
  E_getComponent(AComponent,PropName);
  PValue:=Ord(PropValue);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    SetOrdProp(TObJect(AComponent),PropInfo,PValue);

end;
Function E_SetRealProp(      AComponent : TComponent;
                             PropName   : String;
                             PropValue  : Double) : Boolean;
Var
    PropInfo  : PPropInfo;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
    SetFloatProp(TObJect(AComponent),PropInfo,PropValue);
end;
Function E_SetSetStrProp(      AComponent : TComponent;
                               PropName   : String;
                               PropValue  : String) : Boolean;
Var
    PropInfo  : PPropInfo;
    S         : TIntegerSet;
    TypeInfo  : PPTypeInfo;
    I         : Integer;
    Tp        : String;
begin
  E_getComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
  If Result then
  begin
    While PropValue<>'' do
    begin
      I:=Pos('[',PropValue);
      If I=0 then
        I:=Pos(']',PropValue);
      If I=0 then
        Break;
      Delete(PropValue,I,1);
    end;
    S:=[];
    TypeInfo:=GetTypeData(PropInfo^.PropType^).CompType;
    While PropValue<>'' do
    begin
      I:=Pos(',',PropValue);
      If I=0 then
        I:=Succ(Length(PropValue));
      Tp:=Copy(PropValue,1,Pred(I));
      Delete(PropValue,1,I);
      Include(S,GetEnumValue(TypeInfo^,Tp));
    end;
    SetOrdProp(TObJect(AComponent),PropInfo,Integer(S));
  end;
end;
(*******************************************************************
                            E_EnumProperties
    Enumerate the properties of a component and return them
    in stringlist Proplist where the string is the property name
    and a TEProperty class associated with the property describes
    how to manipulate the property. NOTE that it is expected that
    PropList has not been created.
 *******************************************************************)


Function E_EnumProperties( AComponent : TComponent;
                          Var PropList: TStringList;
                          pSubProp    : string) : Integer;
Var
    PKinds   : TTypeKinds;
    PtData   : PTypeData;
    PList    : PPropList;
    PropInfo : PPropInfo;
    CompProp : TEProperty;
    I,J      : Integer;
    TReal    : Double;
    TInt     : Integer;
    S        : TIntegerSet;
    Ts       : String;
    PCount   : Integer;
    mN,mObj  : string;    // tungli
    AOwner   : TComponent;
    mAddr    : longint;
    OldProp  : string;
    OldResult: integer;
    OldPlist : array[1..c_MaxSize] of byte;
begin
  if pSubProp='' then PropList:=Nil;
  PKinds:=[tkInteger, tkChar, tkEnumeration, tkFloat,tkString,tkSet, tkClass, tkLString];
  //PKinds := [tkInteger, tkChar, tkEnumeration, tkFloat,tkString, tkSet, tkClass, tkMethod, tkWChar, tkLString, tkLWString,tkVariant];
  Result:=GetPropList(AComponent.ClassInfo,PKinds,Nil);
  if (AComponent is TApplication) or (AComponent is TForm) then
  AOwner:=AComponent else AOwner:=AComponent.Owner;
  If Result>0 then
  begin
    GetMem(PList,(Result) * SizeOf(Pointer));
    try
       Result:=GetPropList(AComponent.ClassInfo,PKinds,PList);
       If Result>0 then begin
         if pSubProp='' then PropList:=TStringList.Create;
         for I := 0 to result - 1 do begin
           PCount:=0;
           mObj:=';';    // tungli
           PropInfo := PList^[I];
           CompProp:=TEProperty.Create;
           With CompProp do
           begin
             TypeInfo:=PropInfo^.PropType;
             MinVal:=0;
             MaxVal:=0;
             ClassAddr:=0;
             MaxChars:=0;
             SubType:=0;
             MyClassName:='';
           end;
           With PropInfo^ do
           begin
             CompProp.MyClassName:=PropType^.Name;
             If PropType^.Kind<>tkClass then  {Acts screwy for some reason}
               PtData:=GetTypeData(PropType^);
             Case PropType^.Kind Of
               tkFloat       : begin
                                 TReal:=GetFloatProp(TObJect(AComponent),PropInfo);
                                 With CompProp do
                                 begin
                                   EType:=PROP_REALTYPE;
                                   Case PtData^.FloatType Of
                                     ftSingle : MaxChars:=8;
                                     ftDouble : MaxChars:=16;
                                   else
                                     MaxChars:=20;
                                   end;
                                   PValue:=FloatToStrF(TReal,ffGeneral,MaxChars,0);
                                   MaxChars:=22; {For -&. in edit}
                                 end;
                               end;
               tkChar        : begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 With CompProp do
                                 begin
                                   If (TInt<32) OR (TInt>127) then
                                     PValue:='#'+IntToStr(TInt)
                                   else
                                     PValue:=''+Chr(TInt);
                                   EType:=PROP_CHARTYPE;
                                   MaxChars:=4;
                                 end;
                               end;
               tkString      ,
               tkLString     : With CompProp do
                               begin
                                 EType:=PROP_STRTYPE;
                                 If PropType^.Kind=tkString then
                                   MaxChars:=PtData^.MaxLength;
                                 If PropInfo^.Name='Name' then
                                   MaxChars:=63;
                                 PValue:=UpperCase(PropInfo.Name);
                                 If AComponent IS TDataSet {TTable}then  // tungli
                                 begin
                                   If Pvalue='DATABASENAME' then
                                     EType:=PROP_DBNAMETYPE
                                   else If PValue='INDEXFIELDNAMES' then
                                     EType:=PROP_DBIDXNAMETYPE
                                   else If PValue='TABLENAME' then
                                     EType:=PROP_DBTABNAMETYPE
                                   else if pValue='INDEXNAME' then
                                     EType:=PROP_INDEXNAME;
                                 end
                                 else If (Pvalue='LOOKUPFIELD') OR (PValue='LOOKUPDISPLAY') then
                                   EType:=PROP_DBLOOKUPFIELD;
                                 PValue:=GetStrProp(TObJect(AComponent),PropInfo);
                               end;
               tkEnumeration : begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 With CompProp do If UpperCase(Name)='BOOLEAN' then
                                 begin
                                   If TInt=0 then
                                     PValue:='False'
                                   else
                                     PValue:='True';
                                   EType:=PROP_BOOLTYPE;
                                 end
                                 else
                                 begin
                                   PValue:=GetEnumName(PropType^,TInt);
                                   MinVal:=PtData^.MinValue;
                                   MaxVal:=PtData^.MaxValue;
                                   EType:=PROP_ENUMTYPE;
                                 end;
                               end;
               tkInteger     : With CompProp do
                               begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 PValue:=UpperCase(PropInfo.Name);
                                 If Pos('COLOR',PValue)>0 then
                                 begin
                                   PValue:=ColorToString(TColor(TInt));
                                   EType:=PROP_COLORTYPE;
                                 end
                                 else If Pos('CURSOR',PValue)>0 then
                                 begin
                                   PValue:=CursorToString(TCursor(TInt));
                                   EType:=PROP_CURSORTYPE;
                                 end
                                 else
                                 begin
                                   PValue:=IntToStr(TInt);
                                   EType:=PROP_INTTYPE;
                                   If (PtData<>Nil) then
                                   begin
                                     MinVal:=PtData^.MinValue;
                                     MaxVal:=PtData^.MaxValue;
                                   end;
                                   Case PtData^.OrdType Of
                                     otSByte : MaxChars:=4;
                                     otUByte : MaxChars:=3;
                                     otSWord : MaxChars:=6;
                                     otUWord : MaxChars:=5;
                                     otSLong : MaxChars:=11;
                                   end;
                                 end;
                               end;
               tkSet          : With CompProp do
                                begin
                                  EType:=PROP_SETTYPE;
                                  TypeInfo:=ptData^.CompType;
                                  PtData:=GetTypeData(TypeInfo^);
                                  MinVal:=PtData^.MinValue;
                                  MaxVal:=PtData^.MaxValue;
                                  Integer(S):=GetOrdProp(TObJect(AComponent),PropInfo);
                                  CompProp.PValue := '[';
                                  for J := MinVal to MaxVal do
                                    if J IN S then
                                    begin
                                      if Length(PValue) <> 1 then
                                        PValue := PValue + ',';
                                      PValue := PValue + GetEnumName(TypeInfo^,J);
                                    end;
                                  PValue := PValue + ']';
                                end;
               tkClass        : With CompProp do
                                begin
                                  EType:=PROP_CLASSTYPE;
                                  Ts:=UpperCase(PropType^.Name);
                                  PValue:='('+PropType^.Name+')';
                                  ClassAddr:=GetOrdProp(AComponent,PropInfo);
(*                                If Ts = 'TFONT' then
                                  begin
                                    SubType:=PROP_FONTSUB;

                                    If ClassAddr>0 then begin
                                      PValue:=TFont(ClassAddr).Name;
                                      PCount:=GetPropList(TComponent(ClassAddr).ClassInfo,PKinds,Nil);
                                    end;
                                  end
                                  else *) If Ts = 'TICON' then
                                    SubType:=PROP_ICONSUB
                                  else If Ts = 'TBITMAP' then
                                    SubType:=PROP_BMPSUB
                                  else If Ts = 'TSTRINGS' then
                                    SubType:=PROP_TSTRSUB
                                  else If (Ts='TDATASET') OR (Ts='TTABLE') then
                                  begin
                                    If ClassAddr>0 then begin
                                      PValue:=TDataSet(ClassAddr).Name;
                                      if TDataSet(ClassAddr).Owner<>AOwner then pValue:=TDataSet(ClassAddr).Owner.Name+'.'+PValue;
                                    end else
                                      PValue:='nil';   // tungli
                                    SubType:=PROP_DATASETSUB;
                                  end
                                  else If (Ts='TDATASOURCE') then
                                  begin
                                    If ClassAddr>0 then begin
                                      PValue:=TDataSource(ClassAddr).Name;
                                      if TDataSource(ClassAddr).Owner<>AOwner then pValue:=TDataSource(ClassAddr).Owner.Name+'.'+PValue;
                                    end else
                                      PValue:='nil'; //tungli
                                    SubType:=PROP_DATASOURCESUB;
                                  end
                                  else
                                  begin
                                    // tungli - Is it my component on the Application forms ?
                                    if CompExist(PropType^.Name) then begin
                                      if ClassAddr>0 then begin
                                         mN:=Trim(TComponent(ClassAddr).Name);
                                         IF mN<>'' then begin
                                           SubType:=PROP_COMPONENT;
                                           if TComponent(ClassAddr).Owner<>AOwner then pValue:=TComponent(ClassAddr).Owner.Name+'.'+mN else pValue:=mN;
                                         end;
                                      end else begin
                                        PValue:='nil';
                                        SubType:=PROP_COMPONENT;
                                      end;
                                    end;
                                    if SubType<>PROP_COMPONENT then begin
                                      PCount :=0; {Does It Have Some Properties?}
                                      If ClassAddr>0 then
                                        Try
                                          PCount:=GetPropList(TComponent(ClassAddr).ClassInfo,PKinds,Nil);
                                        Except
                                          PCount:=0;
                                        end;
                                      If PCount<1 then {Just Show It! ???????????????}
                                      begin
                                         EType:=PROP_NOTYPE;
                                         PValue:='*'+PropType^.Name+'*';
                                      end else begin
                                         If Ts = 'TFONT' then
                                           SubType:=PROP_FONTSUB
                                         else EType:=PROP_NOTYPE;
                                         PValue:=PropType^.Name;
                                         mAddr:=ClassAddr;
                                      end;
                                    end;

                                  end;
                                end;
             end;
           end;
           PropList.AddObject(pSubProp+PropInfo^.Name,CompProp);
           if (PCount>0) then begin
             // save
             OldProp:=pSubProp;
             OldResult:=Result;
             if (Result) * SizeOf(Pointer)>c_MaxSize then pSubProp:=''
             else begin
               Move(PList^,OldPlist,(Result) * SizeOf(Pointer));
               FreeMem(PList,(Result) * SizeOf(Pointer));
               // run
               pSubProp:=pSubProp+PropInfo^.Name+'.';
               E_EnumProperties( TComponent(mAddr),PropList,pSubProp); // rekurzia
               // load
               pSubProp:=OldProp;
               Result:=OldResult;
               GetMem(PList,(Result) * SizeOf(Pointer));
               Move(OldPlist,PList^,(Result) * SizeOf(Pointer));
             end;
           end;
         end;
       end;
    finally
      FreeMem(PList,(Result) * SizeOf(Pointer));
      Result:=PropList.count;
    end;
  end;
end;

function CompExist(pClassName:string):boolean; // tungli
var i,j:integer;
    mF:TComponent;
    mParent:TClass;
    mS:String;
begin
  Result:=false;
  pClassName:=UpperCase(pClassName);
  for i:=0 to Application.ComponentCount-1 do begin
    if Application.Components[i] is TForm then begin
      mF:=Application.Components[i];
      if mF=nil then Continue;
      for j:=0 to mF.ComponentCount-1 do begin
        if mF.Components[j]=nil then Continue;
        mS:=UpperCase(mF.Components[j].ClassName);
        if mS=pClassName then begin
          Result:=true;
          exit;
        end else begin
          mParent:=mF.Components[j].ClassParent;
          while mParent<>nil do begin
            mS:=UpperCase(mParent.ClassName); // must save to mS, else chaos
            if mS=pClassName then begin
              Result:=true;
              exit;
            end;
            mParent:=mParent.ClassParent;
          end;
        end;
      end; {for j}
    end;
  end;
end;  {** CompExist **}

(*******************************************************************
                            E_GetEnumList
    Get a string list representing the values of a set type or
    an enumerated type property. NOTE that it is expected AList has
    already been created.
 *******************************************************************)
Function E_GetEnumList(      CompProp : TEProperty;
                             AList    : TStrings) : Boolean;
Var
    I        : Integer;
begin
  AList.Clear;
  Result:=False;
  If CompProp=Nil then
    Exit;
  If CompProp.EType=PROP_BOOLTYPE then
  begin
    AList.Add('True');
    AList.Add('False');
  end else For I:=CompProp.MinVal to CompProp.MaxVal do
    AList.Add(GetEnumName(CompProp.TypeInfo^,I));
  Result:=True;
end;

Procedure GetPropValue (AComponent : TComponent; PropName:string; var CompProp:TEProperty);
Var
    PKinds   : TTypeKinds;
    PtData   : PTypeData;
    PropInfo : PPropInfo;
    J        : Integer;
    TReal    : Double;
    TInt     : Integer;
    S        : TIntegerSet;
    Ts       : String;
    PCount   : Integer;
    mObj  : string;    // tungli
    AOwner   : TComponent;
begin
  PKinds:=[tkInteger, tkChar, tkEnumeration, tkFloat,tkString, tkSet, tkClass, tkLString];
{    TTypeKind = (tkUnknown, tkInteger, tkChar, tkEnumeration, tkFloat,
    tkString, tkSet, tkClass, tkMethod, tkWChar, tkLString, tkLWString,
    tkVariant); }
  E_getComponent(AComponent,PropName);
  if (AComponent is TApplication) or (AComponent is TForm) then
    AOwner:=AComponent else AOwner:=AComponent.Owner;
           PropInfo :=GetPropInfo(AComponent.ClassInfo,PropName);
           if PropInfo=nil then exit;
           mObj:=';';    // tungli
           With CompProp do
           begin
             TypeInfo:=PropInfo^.PropType;
             MinVal:=0;
             MaxVal:=0;
             ClassAddr:=0;
             MaxChars:=0;
             SubType:=0;
             MyClassName:='';
           end;
           With PropInfo^ do begin
             CompProp.MyClassName:=PropType^.Name;
             If PropType^.Kind<>tkClass then  {Acts screwy for some reason}
               PtData:=GetTypeData(PropType^);
             Case PropType^.Kind Of
               tkFloat       : begin
                                 TReal:=GetFloatProp(TObJect(AComponent),PropInfo);
                                 With CompProp do
                                 begin
                                   EType:=PROP_REALTYPE;
                                   Case PtData^.FloatType Of
                                     ftSingle : MaxChars:=8;
                                     ftDouble : MaxChars:=16;
                                   else
                                     MaxChars:=20;
                                   end;
                                   PValue:=FloatToStrF(TReal,ffGeneral,MaxChars,0);
                                   MaxChars:=22; {For -&. in edit}
                                 end;
                               end;
               tkChar        : begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 With CompProp do
                                 begin
                                   If (TInt<32) OR (TInt>127) then
                                     PValue:='#'+IntToStr(TInt)
                                   else
                                     PValue:=''+Chr(TInt);
                                   EType:=PROP_CHARTYPE;
                                   MaxChars:=4;
                                 end;
                               end;
               tkString      ,
               tkLString     : With CompProp do
                               begin
                                 EType:=PROP_STRTYPE;
                                 If PropType^.Kind=tkString then
                                   MaxChars:=PtData^.MaxLength;
                                 If PropInfo^.Name='Name' then
                                   MaxChars:=63;
                                 PValue:=UpperCase(PropInfo.Name);
                                 If AComponent IS TDataSet {TTable}then  // tungli
                                 begin
                                   If Pvalue='DATABASENAME' then
                                     EType:=PROP_DBNAMETYPE
                                   else If PValue='INDEXFIELDNAMES' then
                                     EType:=PROP_DBIDXNAMETYPE
                                   else If PValue='TABLENAME' then
                                     EType:=PROP_DBTABNAMETYPE
                                   else if pValue='INDEXNAME' then
                                     EType:=PROP_INDEXNAME;
                                 end
                                 else If (Pvalue='LOOKUPFIELD') OR (PValue='LOOKUPDISPLAY') then
                                   EType:=PROP_DBLOOKUPFIELD;
                                 PValue:=GetStrProp(TObJect(AComponent),PropInfo);
                               end;
               tkEnumeration : begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 With CompProp do If UpperCase(Name)='BOOLEAN' then
                                 begin
                                   If TInt=0 then
                                     PValue:='False'
                                   else
                                     PValue:='True';
                                   EType:=PROP_BOOLTYPE;
                                 end
                                 else
                                 begin
                                   PValue:=GetEnumName(PropType^,TInt);
                                   MinVal:=PtData^.MinValue;
                                   MaxVal:=PtData^.MaxValue;
                                   EType:=PROP_ENUMTYPE;
                                 end;
                               end;
               tkInteger     : With CompProp do
                               begin
                                 TInt:=GetOrdProp(TObJect(AComponent),PropInfo);
                                 PValue:=UpperCase(PropInfo.Name);
                                 If Pos('COLOR',PValue)>0 then
                                 begin
                                   PValue:=ColorToString(TColor(TInt));
                                   EType:=PROP_COLORTYPE;
                                 end
                                 else If Pos('CURSOR',PValue)>0 then
                                 begin
                                   PValue:=CursorToString(TCursor(TInt));
                                   EType:=PROP_CURSORTYPE;
                                 end
                                 else
                                 begin
                                   PValue:=IntToStr(TInt);
                                   EType:=PROP_INTTYPE;
                                   If (PtData<>Nil) then
                                   begin
                                     MinVal:=PtData^.MinValue;
                                     MaxVal:=PtData^.MaxValue;
                                   end;
                                   Case PtData^.OrdType Of
                                     otSByte : MaxChars:=4;
                                     otUByte : MaxChars:=3;
                                     otSWord : MaxChars:=6;
                                     otUWord : MaxChars:=5;
                                     otSLong : MaxChars:=11;
                                   end;
                                 end;
                               end;
               tkSet          : With CompProp do
                                begin
                                  EType:=PROP_SETTYPE;
                                  TypeInfo:=ptData^.CompType;
                                  PtData:=GetTypeData(TypeInfo^);
                                  MinVal:=PtData^.MinValue;
                                  MaxVal:=PtData^.MaxValue;
                                  Integer(S):=GetOrdProp(TObJect(AComponent),PropInfo);
                                  CompProp.PValue := '[';
                                  for J := MinVal to MaxVal do
                                    if J IN S then
                                    begin
                                      if Length(PValue) <> 1 then
                                        PValue := PValue + ',';
                                      PValue := PValue + GetEnumName(TypeInfo^,J);
                                    end;
                                  PValue := PValue + ']';
                                end;
               tkClass        : With CompProp do
                                begin
                                  EType:=PROP_CLASSTYPE;
                                  Ts:=UpperCase(PropType^.Name);
                                  PValue:='('+PropType^.Name+')';
                                  ClassAddr:=GetOrdProp(AComponent,PropInfo);
                                  If Ts = 'TFONT' then
                                  begin
                                    SubType:=PROP_FONTSUB;
                                    If ClassAddr>0 then begin
                                      PValue:=TFont(ClassAddr).Name;
                                    end;
                                  end
                                  else If Ts = 'TICON' then
                                    SubType:=PROP_ICONSUB
                                  else If Ts = 'TBITMAP' then
                                    SubType:=PROP_BMPSUB
                                  else If Ts = 'TSTRINGS' then begin
                                    SubType:=PROP_TSTRSUB;
(*                  PROP_TSTRSUB:
                  begin
                    if SedForm=nil then Application.CreateForm(TSedForm,SedForm); // tungli
                    SedForm.StrList.Lines.Assign(TStrings(CompProp.ClassAddr));
                    If (SedForm.ShowModal=mrOk) then TStrings(CompProp.ClassAddr).Assign(SedForm.StrList.Lines);
                  end;
  *)                                  CompProp.pValue:=TStrings(CompProp.ClassAddr).text;
                                  end
                                  else If (Ts='TDATASET') OR (Ts='TTABLE') then
                                  begin
                                    If ClassAddr>0 then begin
                                      PValue:=TDataSet(ClassAddr).Name;
                                      if TDataSet(ClassAddr).Owner<>AOwner then pValue:=TDataSet(ClassAddr).Owner.Name+'.'+PValue;
                                    end else
                                      PValue:='nil';   // tungli
                                    SubType:=PROP_DATASETSUB;
                                  end
                                  else If (Ts='TDATASOURCE') then
                                  begin
                                    If ClassAddr>0 then begin
                                      PValue:=TDataSource(ClassAddr).Name;
                                      if TDataSource(ClassAddr).Owner<>AOwner then pValue:=TDataSource(ClassAddr).Owner.Name+'.'+PValue;
                                    end else
                                      PValue:='nil'; //tungli
                                    SubType:=PROP_DATASOURCESUB;
                                  end
                                  else
                                  begin
                                    // tungli - Is it my component on the Application forms ?
                                    if CompExist(PropType^.Name) then begin
                                        if ClassAddr>0 then begin
                                           PValue:=TComponent(ClassAddr).Name;
                                           if TComponent(ClassAddr).Owner<>AOwner then pValue:=TComponent(ClassAddr).Owner.Name+'.'+PValue;
                                        end else PValue:='nil';
                                        SubType:=PROP_COMPONENT;
                                    end;
                                    if SubType<>PROP_COMPONENT then begin
                                      PValue:='';
                                      PCount :=0; {Does It Have Some Properties?}
                                      If ClassAddr>0 then
                                        Try
                                          PCount:=GetPropList(TComponent(ClassAddr).ClassInfo,PKinds,Nil);
                                        Except
                                          PCount:=0;
                                        end;
                                      If PCount<1 then {Just Show It!}
                                      begin
                                         if SubType<>PROP_COMPONENT then begin
                                           EType:=PROP_NOTYPE;
                                           PValue:='*'+PropType^.Name+'*';
                                        end;
                                      end;
                                    end;

                                  end;
                                end;
             end;
           end;
end;

function  FindComp(AOwnerForm:TComponent; CompName:String):TComponent;
var mF:TComponent;
    mN:string;
begin
  Result:=nil;
  if Pos('.',CompName)>0 then begin
    mN:=Copy(CompName,1,Pos('.',CompName)-1);
    if mN='' then exit;
    mF:=Application.FindComponent(mN); // mF is form
    if mF<>nil then begin
      mN:=Trim(Copy(CompName,Pos('.',CompName)+1,500));
      if mN='' then exit;
      Result:=mF.FindComponent(mN);
    end;
  end else Result:=AOwnerForm.FindComponent(CompName);
end;

procedure SetPropValue(AComponent:TComponent; PropName:string;xValue:string);
// SetPropValue(Query1,'SQL','Select *'+#13+'From TBanka')
// SetPropValue(Edit1,'Font.Style','[fsBold,fsItalic]')
// SetPropValue(Edit1,'Font.Color','clRed')
Var
    OldValue : String;
    S       : String;
    TInt    : Integer;
    Tf      : Extended;
    TBool   : Boolean;
    CAddr   : LongInt;
    I,j,k     : Integer;
    mT      : TComponent; // tungli temp value
    mComboEnum: TStringList;
    CompProp: TEProperty;
    PropInfo : PPropInfo;
    AOwner:TComponent;
begin
 try
    E_GetComponent(AComponent,PropName);
    OldValue:='';
    CompProp:=TEProperty.Create;
    if (AComponent is TApplication) or (AComponent is TForm) then
      AOwner:=AComponent else AOwner:=AComponent.Owner;
    PropInfo :=GetPropInfo(AComponent.ClassInfo,PropName);
    if propInfo=nil then exit;
    With CompProp do
    begin
       TypeInfo:=PropInfo^.PropType;
       MinVal:=0;
       MaxVal:=0;
       ClassAddr:=0;
       MaxChars:=0;
       SubType:=0;
       MyClassName:='';
     end;
    GetPropValue(AComponent,PropName,CompProp);
    CAddr:=longint(nil);
    if (CompProp=nil) or (CompProp.Etype=PROP_NOTYPE) then exit;
    If CompProp<>nil then begin
      OldValue:=CompProp.PValue;
      CompProp.PValue:='nil';
      If CompProp.EType=PROP_STRTYPE then begin
        If E_SetStrProp(AComponent,PropName,xValue) then CompProp.PValue:=xValue;
      end else
        If CompProp.EType=PROP_CHARTYPE then begin
          S:=StripAllChr(xValue,#32);
          If S='' then TInt:=0
          else begin
            If S[1]='#' then begin
              Delete(S,1,1);
              TInt:=StrToInt(S);
            end else TInt:=Ord(S[1]);
          end;
          If E_SetIntProp(AComponent,PropName,TInt) then CompProp.PValue:=xValue;
        end else
          If CompProp.EType=PROP_BOOLTYPE then begin
            If E_SetBoolProp(AComponent,PropName,UpperCase(Trim(xValue))='TRUE') then CompProp.PValue:=xValue
          end else
            If CompProp.EType=PROP_INTTYPE then begin
              TInt:=StrToInt(xValue);
              With CompProp do
                if (TInt < MinVal) or (TInt > MaxVal) then;//raise ECompEditError.CreateResFmt(SOutOfRange, [MinVal,MaxVal]);
              If E_SetIntProp(AComponent,PropName,TInt) then CompProp.PValue:=intToStr(TInt);
            end else
              If CompProp.EType=PROP_REALTYPE then begin
                Tf:=StrToFloat(xValue);
                If E_SetRealProp(AComponent,PropName,Tf) then CompProp.PValue:=xValue;
              end else
                If CompProp.EType IN [PROP_ENUMTYPE,PROP_MODALTYPE] then begin
                  mComboEnum:=TStringList.Create;
                  mComboEnum.Clear;
                  case CompProp.EType of
                    PROP_ENUMTYPE   : E_GetEnumList(CompProp,mComboEnum);
                    PROP_MODALTYPE  : For I:=MrNone to MrNo do mComboEnum.Add(MrArray[I]);
                  end;
                  k:=-1;
                  for i:=0 to mComboEnum.Count-1 do begin
                    if UpperCase(mComboEnum.strings[i])=UpperCase(xValue) then BEGIN k:=i; break; END;
                  end;
                  mComboEnum.Free;
                  If k>-1 then if E_SetIntProp(AComponent,PropName,k) then CompProp.PValue:=xValue;
             end else
                  If CompProp.EType=PROP_SETTYPE then begin
                    CompProp.PValue:=xValue;
                    If NOT E_SetSetStrProp(AComponent,PropName,xValue) then CompProp.PValue:=xValue;
                  end else
                    If CompProp.EType=PROP_COLORTYPE then begin
                      If E_SetIntProp(AComponent,PropName,StringToColor(xValue)) then CompProp.PValue:=xValue;
                    end else
                      If CompProp.EType=PROP_CURSORTYPE then begin
                        If E_SetIntProp(AComponent,PropName,StringToCursor(xValue)) then CompProp.PValue:=xValue;
                      end else
                        If CompProp.EType IN [PROP_INDEXNAME,PROP_DBNAMETYPE,PROP_DBIDXNAMETYPE,
                                 PROP_DBTABNAMETYPE,PROP_DBLOOKUPFIELD] then begin
                          if (Trim(xValue)<>'') and (CompProp.EType=PROP_INDEXNAME) then E_SetStrProp(AComponent,'IndexFieldNames','');
                          if (Trim(xValue)<>'') and (CompProp.EType=PROP_DBIDXNAMETYPE) then E_SetStrProp(AComponent,'IndexName','');
                          if (Trim(xValue)<>OldValue) and ((CompProp.EType=PROP_DBNAMETYPE) or (CompProp.EType=PROP_DBTABNAMETYPE)) then begin
                            E_SetStrProp(AComponent,'IndexFieldNames','');
                            E_SetStrProp(AComponent,'IndexName','');
                            if (CompProp.EType=PROP_DBNAMETYPE) then E_SetStrProp(AComponent,'TableName','');
                          end;
                          If E_SetStrProp(AComponent,PropName,xValue) then CompProp.PValue:=xValue;
                        end else
                          If CompProp.EType = PROP_CLASSTYPE then begin    //tungli begin
                            if CompProp.SubType=PROP_TSTRSUB then  TStrings(CompProp.ClassAddr).text:=xValue;
(*                  PROP_TSTRSUB:
                  begin
                    if SedForm=nil then Application.CreateForm(TSedForm,SedForm); // tungli
                    SedForm.StrList.Lines.Assign(TStrings(CompProp.ClassAddr));
                    If (SedForm.ShowModal=mrOk) then TStrings(CompProp.ClassAddr).Assign(SedForm.StrList.Lines);
                  end;
  *)                         if CompProp.SubType=PROP_COMPONENT then begin
                              TBool:=false;
                              mT:=FindComp(Aowner,xValue);
                              if mT<>nil then begin
                                CAddr:=Longint(mT);
                                TBool:=E_SetIntProp(AComponent,PropName,CAddr);
                                 if Tbool then begin
                                   CompProp.PValue:=mT.Name;
                                   CompProp.ClassAddr:=CAddr;
                                 end;
                              end;
                              if not TBool then begin
                                TBool:=E_SetIntProp(AComponent,PropName,longint(nil));
                                if TBool then begin
                                  CompProp.PValue:='nil';
                                  CompProp.ClassAddr:=Longint(nil);
                                end;
                              end;
                            TBool:=false;
                          end;                             // tungli end
                          If CompProp.SubType=PROP_DATASETSUB then begin
                            CAddr:=Longint(nil);
                            TBool:=False;
                            mT:=FindComp(AOwner,xValue);
                            if mT<>nil then begin
                              if mT is TDataSet then begin
                                TBool:=True;
                                CAddr:=Longint(mT);
                              end;
                            end;
                            If NOT TBool then
                              With Session do
                                for I:= 0 to DatabaseCount - 1 do
                                  With Databases[I] do
                                    For J:=0 to DataSetCount-1 do
                                      If CompareText(xValue,DataSets[I].Name)=0 then begin
                                        CAddr:=Longint(DataSets[I]);
                                        Break;
                                      end;
                            If E_SetIntProp(AComponent,PropName,CAddr) then begin
                              CompProp.PValue:=xValue;
                              CompProp.ClassAddr:=CAddr;
                            end;
                          end else
                            If CompProp.SubType=PROP_DATASOURCESUB then begin
                              TBool:=True;
                              CAddr:=longint(nil);
                              mT:=FindComp(AOwner,xValue);
                              if (mT<> nil) and (mT is TDataSource) then begin
                                TBool:=true;
                                CAddr:=Longint(mT);
                              end;
                              TBool:=E_SetIntProp(AComponent,PropName,CAddr);
                              If TBool then begin
                                CompProp.PValue:=xValue;
                                CompProp.ClassAddr:=CAddr;
                              end;
                            end;
                          end;
                        end;
finally
  CompProp.Free;
end;
end;  {** SetPropValue **}

function GetPropValueStr(AComponent : TComponent; PropName:string):string;
var
    CompProp: TEProperty;
    PropInfo : PPropInfo;
begin
    E_GetComponent(AComponent,PropName);
    CompProp:=TEProperty.Create;
    PropInfo :=GetPropInfo(AComponent.ClassInfo,PropName);
    if propInfo=nil then exit;
    With CompProp do
    begin
       TypeInfo:=PropInfo^.PropType;
       MinVal:=0;
       MaxVal:=0;
       ClassAddr:=0;
       MaxChars:=0;
       SubType:=0;
       MyClassName:='';
     end;
    GetPropValue(AComponent,PropName,CompProp);
    Result:=CompProp.PValue;
    MyLastPropType:=CompProp.EType;
    //MyLastClassName:=CompProp.MyClassName;
end;


function ExistObj(ObjName:string;DefaultFormName:string='SELF'):boolean;
// ExistObj('Application');
// ExistObj('Session');
// ExistObj('Screen');
// ExistObj('Form1.Query1');
// ExistObj('Button1');
var AComponent:TComponent;
    DefaultForm:Tform;
begin
  ObjName:=AnsiUpperCase(Trim(ObjName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  result:=false;
  if (DefaultFormName='SELF') or (DefaultFormName='')  then DefaultForm:=Screen.ActiveForm
  else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then exit;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else  exit;
  end;
  ObjName:=ObjName+'.Name';
  Result:=nil<>ParseProp(DefaultForm,ObjName);
end;

function ClassNameObj(ObjName:string;DefaultFormName:string='SELF'):string;
// ExistObj('Application');
// ExistObj('Session');
// ExistObj('Screen');
// ExistObj('Form1.Query1');
// ExistObj('Button1');
var AComponent:TComponent;
    DefaultForm:Tform;
begin
  ObjName:=AnsiUpperCase(Trim(ObjName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  result:='';
  if (DefaultFormName='SELF') or (DefaultFormName='')  then DefaultForm:=Screen.ActiveForm
  else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then exit;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else  exit;
  end;
  ObjName:=ObjName+'.Name';
  AComponent:=ParseProp(DefaultForm,ObjName);
  if AComponent<>nil then result:=AComponent.ClassName;
end;

function GetComponent(ObjName:string;DefaultFormName:string='SELF'):Tcomponent;
var AComponent:TComponent;
    DefaultForm:Tform;
begin
  ObjName:=AnsiUpperCase(Trim(ObjName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  result:=nil;
  if (DefaultFormName='SELF') or (DefaultFormName='')  then DefaultForm:=Screen.ActiveForm
  else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then exit;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else  exit;
  end;
  ObjName:=ObjName+'.Name';
  result:=ParseProp(DefaultForm,ObjName);
end;

function ExistProp(PropName:string; DefaultFormName:string='SELF'):boolean;
// ExistProp('Application.Name');
// ExistObj('Session.name');
//// ExistObj('Screen.Cursor'); TOTO NEIDE
// ExistObj('Form1.Query1.SQL');
// ExistObj('Button1.Caption');
var AComponent:TComponent;
    PropInfo:PPropInfo;
    DefaultForm:Tform;
begin
  result:=false;
  PropName:=AnsiUpperCase(Trim(propName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  if (DefaultFormName='SELF') or (DefaultFormName='')  then begin
     if Pos('SELF.',PropName)=1 then delete(propName,1,5);
     DefaultForm:=Screen.ActiveForm;
  end else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then exit;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else exit;
  end;
  AComponent:=ParseProp(DefaultForm,PropName);
  if AComponent=nil then exit;
  E_GetComponent(AComponent,PropName);
  Result:=E_GetPropInfo(AComponent,PropName,PropInfo);
end;

function GetProp(PropName:string; DefaultFormName:string='SELF'):string;
var AComponent:TComponent;
    mProp:String;
    DefaultForm:Tform;
// Str:=getProp('Form2.Button1.Font.Style','Form1');
// Str:=getProp('Button2.Width','Form2');
// Str:=getProp('Caption');  // DefaultFormName is Screen.ActiveForm.Name
begin
  mProp:=PropName;
  PropName:=AnsiUpperCase(Trim(PropName));
  result:='';
  if (DefaultFormName='SELF') or (DefaultFormName='')  then begin
     if Pos('SELF.',PropName)=1 then delete(propName,1,5);
    DefaultForm:=Screen.ActiveForm;
  end else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then begin ShowMessage(DefaultFormName+' - form is error'); exit; end;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else begin ShowMessage(DefaultFormName+' - form is error'); exit; end;
  end;
  AComponent:=ParseProp(DefaultForm,PropName);
  if AComponent=nil then begin ShowMessage(mProp+'-Error'); exit; end;
  result:=GetPropValueStr(AComponent,PropName);
  if result<>'' then begin
    if (Pos('*T',result)=1) and (result[length(result)]='*') then result:='';
  end;
end;


procedure SetProp(PropName,xValue:string; DefaultFormName:string='SELF');
var AComponent:TComponent;
    mProp:String;
    DefaultForm:Tform;
// SetProp('Form2.Button1.Width' , '100','Form1');
// SetProp('Button2.Width' , '100');
// SetProp('Query1.SQL' , 'Select *'+#13+'From TBanka')
// SetProp('Edit1.Font.Style' , '[fsBold,fsItalic]')
// SetProp('Edit1.Font.Color' , 'clRed')
// SetProp('Font.Color' , 'clRed');
// SetProp('Form1.Font.Color' , 'clRed');
// SetProp('Screen.Cursor' , 'crHourGlass');  TOTO NEIDE
begin
  PropName:=AnsiUpperCase(Trim(propName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  if (DefaultFormName='SELF') or (DefaultFormName='') then begin
     if Pos('SELF.',PropName)=1 then delete(propName,1,5);
    DefaultForm:=Screen.ActiveForm;
  end else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then begin ShowMessage(DefaultFormName+' - form is error'); exit; end;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else begin ShowMessage(DefaultFormName+' - form is error'); exit; end;
  end;
  mProp:=PropName;
  AComponent:=ParseProp(DefaultForm,PropName);
  if AComponent=nil then begin ShowMessage(mProp+'-Error'); exit; end;
  SetPropValue(AComponent,PropName,xValue);
end;

function ParseProp(DefaultForm:TForm; var PropName:string):TComponent;
var  mT:String;
     mObj:TComponent;
begin
if Pos('.',PropName)=0 then begin result:=DefaultForm; exit; end;
  mObj:=nil;
  while Pos(' ',PropName)>0 do delete(PropName, Pos(' ',PropName), 1);
  while Pos('..',PropName)>0 do delete(PropName, Pos('..',PropName), 1);
  while Pos('.',PropName)>0 do begin
    mT:=Trim(copy(PropName,1,Pos('.',PropName)-1));
    delete(PropName,1,Pos('.',PropName));
    Result:=mObj;
    if result is TApplication then begin
      mObj:=Application.FindComponent(mT);
      if mObj<>nil then begin Result:=mObj; Continue; end
      else exit;
    end;
    if result is TForm then begin
      mObj:=Result.FindComponent(mT);
      if mObj<>nil then begin result:=mObj; exit; end
      else begin PropName:=mT+'.'+PropName; exit; end;
    end;
    if (Result=nil) then begin
      if (AnsiUpperCase(mT)='APPLICATION') then begin
        Result:=Application; continue;
      end;
      if (AnsiUpperCase(mT)='SESSION') then begin
        Result:=SESSION; exit;
      end;
      if (AnsiUpperCase(mT)='SCREEN') then begin
        Result:=TScreen(SCREEN); exit;
      end;
      Result:=Application.FindComponent(mT);
      if Result=nil then begin
        Result:=DefaultForm.FindComponent(mT);
        if Result<>nil then exit;
      end else mObj:=Result;
    end;
    if Result=nil then exit;
  end;
end;

function GetCompProp(AComponent : TComponent; PropName:string):TEProperty;
  //GetCompProp('Query1','SQL')
var
    CompProp: TEProperty;
    PropInfo : PPropInfo;
begin
    result:=nil;
    E_GetComponent(AComponent,PropName);
    CompProp:=TEProperty.Create;
    PropInfo :=GetPropInfo(AComponent.ClassInfo,PropName);
    if propInfo=nil then exit;
    With CompProp do
    begin
       TypeInfo:=PropInfo^.PropType;
       MinVal:=0;
       MaxVal:=0;
       ClassAddr:=0;
       MaxChars:=0;
       SubType:=0;
       MyClassName:='';
     end;
   (*
    if compProp.Etype=PROP_CLASSTYPE then begin
     case CompProp.Subtype of
       PROP_TSTRSUB:begin
         if metodName='CLEAR' then TStrings(CompProp.ClassAddr).Clear;
       end;
     end; {case}
   end;
   *)
   result:=CompProp;
end;

function SuperParse(PropName:string; DefaultFormName:string='SELF'):string;
//Object.Property>Metoda;
//Object.Property;
var AComponent:TComponent;
    PropInfo:PPropInfo;
    DefaultForm:Tform;
begin
  result:='';
  PropName:=AnsiUpperCase(Trim(propName));
  DefaultFormName:=AnsiUpperCase(trim(DefaultFormName));
  if (DefaultFormName='SELF') or (DefaultFormName='')  then begin
     if Pos('SELF.',PropName)=1 then delete(propName,1,5);
     DefaultForm:=Screen.ActiveForm;
  end else begin
     AComponent:=Application.FindComponent(DefaultFormName);
    if AComponent=nil then exit;
    if AComponent is Tform then DefaultForm:=TForm(AComponent)
      else exit;
  end;
  AComponent:=ParseProp(DefaultForm,PropName);
  if AComponent=nil then PropName:='' else begin
    if Pos('.',PropName)>0 then  propName[Pos('.',PropName)]:='>';
    PropName:=AComponent.Name+'.'+PropName+';';
  end;
  result:=PropName;
end;

{-------------------------END OF FILE---------------------------------}
end.


